@@ -10,7 +10,7 @@ class PluginException(Exception): pass
 
 def load_all(config):
     plugins = config.get('ckan.plugins', '')
-    log.info("Loading plugins: %s" % plugins)
+    log.debug("Loading plugins: %s" % plugins)
     for plugin in plugins.split():
         for entry_point in iter_entry_points(group=GROUP_NAME, name=plugin):
             load(plugin, entry_point, config)
@@ -35,4 +35,31 @@ def find_methods(method_name):
         if hasattr(v, method_name):
             yield getattr(v, method_name)
         else:
-            log.debug("%s has no method %s" % (k.name, method_name))
+            pass
+            #log.debug("%s has no method %s" % (k.name, method_name))
+
+
+
+#####  Pylons monkey-patch
+
+from pylons.wsgiapp import PylonsApp
+import pkg_resources
+
+log.info("Monkey-patching Pylons to allow loading of controllers via entry point mechanism")
+
+find_controller_generic = PylonsApp.find_controller
+
+# This is from pylons 1.0 source, will monkey-patch into 0.9.7
+def find_controller(self, controller):
+    if controller in self.controller_classes:
+        return self.controller_classes[controller]
+
+    # Check to see if its a dotted name
+    if '.' in controller or ':' in controller:
+        mycontroller = pkg_resources.EntryPoint.parse('x=%s' % controller).load(False)
+        self.controller_classes[controller] = mycontroller
+        return mycontroller
+
+    return find_controller_generic(self, controller)
+
+PylonsApp.find_controller = find_controller
