@@ -1,7 +1,7 @@
 import cgi
 
 from paste.urlparser import PkgResourcesParser
-from pylons import request
+from pylons import request, tmpl_context as c
 from pylons.controllers.util import forward
 from pylons.middleware import error_document_template
 from webhelpers.html.builder import literal
@@ -29,13 +29,10 @@ class ErrorController(BaseController):
         if original_request.path.startswith('/api'):
             return original_response.body
         # Otherwise, decorate original response with error template.
-        ckan_template = render('error_document_template.html')
-        content = literal(original_response.body) or cgi.escape(request.GET.get('message', ''))
-        page = ckan_template % \
-            dict(prefix=request.environ.get('SCRIPT_NAME', ''),
-                 code=cgi.escape(request.GET.get('code', str(original_response.status_int))),
-                 message=content)
-        return page
+        c.content = literal(original_response.unicode_body) or cgi.escape(request.GET.get('message', ''))
+        c.prefix=request.environ.get('SCRIPT_NAME', ''),
+        c.code=cgi.escape(request.GET.get('code', str(original_response.status_int))),
+        return render('error_document_template.html')
 
     def img(self, id):
         """Serve Pylons' stock images"""
