@@ -28,6 +28,19 @@ class BaseApiController(BaseController):
     content_type_html = 'text/html;charset=utf-8'
     content_type_json = 'application/json;charset=utf-8'
 
+    def __call__(self, environ, start_response):
+        self._identify_user()
+        if not self.authorizer.am_authorized(c, model.Action.SITE_READ, model.System):
+            response_msg = self._finish(403, _('Not authorized to see this page'))
+            # Call start_response manually instead of the parent __call__
+            # because we want to end the request instead of continuing.
+            response_msg = response_msg.encode('utf8')
+            body = '%i %s' % (response.status_int, response_msg)
+            start_response(body, response.headers.items())
+            return [response_msg]
+        else:
+            return BaseController.__call__(self, environ, start_response)
+
     @classmethod
     def _ref_package(cls, package):
         assert cls.ref_package_by in ['id', 'name']
@@ -38,10 +51,6 @@ class BaseApiController(BaseController):
         assert cls.ref_group_by in ['id', 'name']
         return getattr(group, cls.ref_group_by)
 
-    @classmethod
-    def _ref_harvest_source(cls, harvest_source):
-        return getattr(harvest_source, 'id')
-
     @classmethod
     def _list_package_refs(cls, packages):
         return [getattr(p, cls.ref_package_by) for p in packages]
@@ -95,6 +104,20 @@ class BaseApiController(BaseController):
         return self._finish(status_int, response_data,
                             content_type=content_type)
 
+    def _finish_not_authz(self):
+        response_data = _('Access denied')
+        return self._finish(status_int=403,
+                            response_data=response_data,
+                            content_type='json')
+
+    def _finish_not_found(self, extra_msg=None):
+        response_data = _('Not found')
+        if extra_msg:
+            response_data = '%s - %s' % (response_data, extra_msg)
+        return self._finish(status_int=404,
+                            response_data=response_data,
+                            content_type='json')
+
     def _wrap_jsonp(self, callback, response_msg):
         return '%s(%s);' % (callback, response_msg)
 
@@ -134,21 +157,31 @@ class BaseRestController(BaseApiController):
             revs = model.Session.query(model.Revision).all()
             return self._finish_ok([rev.id for rev in revs])
         elif register == u'package' and not subregister:
-            query = ckan.authz.Authorizer().authorized_query(self._get_username(), model.Package)
-            packages = query.all()
-            response_data = self._list_package_refs(packages)
-            return self._finish_ok(response_data)
+            # see /apiv1/PackageController
+            raise NotImplementedError
         elif register == u'package' and subregister == 'relationships':
-            #TODO authz stuff for this and related packages
             pkg = self._get_pkg(id)
             if not pkg:
-                response.status_int = 404
-                return 'First package named in request was not found.'
-            relationships = pkg.get_relationships()
+                return self._finish_not_found('First package named in request was not found.')
+            relationships = ckan.authz.Authorizer().\
+                            authorized_package_relationships(\
+                c.user, pkg, action=model.Action.READ)
             response_data = [rel.as_dict(package=pkg, ref_package_by=self.ref_package_by) for rel in relationships]
             return self._finish_ok(response_data)
+        elif register == u'package' and subregister == 'revisions':
+            pkg = self._get_pkg(id)
+            if pkg is None:
+                return self._finish_not_found()
+            if not self._check_access(pkg, model.Action.READ):
+                return self._finish_not_authz()
+            revision_dicts = []
+            for revision, object_revisions in pkg.all_related_revisions:
+                revision_dicts.append(model.revision_as_dict(revision,
+                                                             include_packages=False))
+            return self._finish_ok(revision_dicts)
         elif register == u'group':
-            groups = model.Session.query(model.Group).all() 
+            query = ckan.authz.Authorizer().authorized_query(c.user, model.Group)
+            groups = query.all() 
             response_data = self._list_group_refs(groups)
             return self._finish_ok(response_data)
         elif register == u'tag':
@@ -164,25 +197,6 @@ class BaseRestController(BaseApiController):
             licenses = LicenseRegister().values()
             response_data = [l.as_dict() for l in licenses]
             return self._finish_ok(response_data)
-        elif register == u'harvestsource':
-            filter_kwds = {}
-            if id == 'publisher':
-                filter_kwds['publisher_ref'] = subregister
-            objects = model.HarvestSource.filter(**filter_kwds)
-            response_data = [o.id for o in objects]
-            return self._finish_ok(response_data)
-        elif register == u'harvestingjob':
-            filter_kwds = {}
-            if id == 'status':
-                filter_kwds['status'] = subregister.lower().capitalize()
-            objects = model.HarvestingJob.filter(**filter_kwds)
-            response_data = [o.id for o in objects]
-            return self._finish_ok(response_data)
-        elif register == u'harvesteddocument':
-            filter_kwds = {}
-            objects = model.HarvestedDocument.filter(**filter_kwds)
-            response_data = [o.guid for o in objects]
-            return self._finish_ok(response_data)
         else:
             response.status_int = 400
             return gettext('Cannot list entity of this type: %s') % register
@@ -193,16 +207,10 @@ class BaseRestController(BaseApiController):
             # Todo: Implement access control for revisions.
             rev = model.Session.query(model.Revision).get(id)
             if rev is None:
-                response.status_int = 404
-                return ''
-            response_data = {
-                'id': rev.id,
-                'timestamp': model.strftimestamp(rev.timestamp),
-                'author': rev.author,
-                'message': rev.message,
-                'packages': self._list_package_refs(rev.packages),
-            }
-            return self._finish_ok(response_data)
+                return self._finish_not_found()
+            rev_dict = model.revision_as_dict(rev, include_packages=True,
+                                              ref_package_by=self.ref_package_by)
+            return self._finish_ok(rev_dict)
         elif register == u'changeset':
             from ckan.model.changeset import ChangesetRegister
             changesets = ChangesetRegister()
@@ -210,8 +218,7 @@ class BaseRestController(BaseApiController):
             #if not self._check_access(changeset, model.Action.READ):
             #    return ''
             if changeset is None:
-                response.status_int = 404
-                return ''            
+                return self._finish_not_found()
             response_data = changeset.as_dict()
             return self._finish_ok(response_data)
         elif register == u'package' and not subregister:
@@ -234,90 +241,39 @@ class BaseRestController(BaseApiController):
             if not pkg2:
                 response.status_int = 404
                 return 'Second package named in address was not found.'
-            if subregister == 'relationships':
-                relationships = pkg1.get_relationships_with(pkg2)
-            else:
-                relationships = pkg1.get_relationships_with(pkg2,
-                                                            type=subregister)
-                if not relationships:
-                    response.status_int = 404
-                    return 'Relationship "%s %s %s" not found.' % \
-                           (id, subregister, id2)
+            rel_type = subregister if subregister != 'relationships' else None
+
+            relationships = ckan.authz.Authorizer().\
+                            authorized_package_relationships(\
+                c.user, pkg1, pkg2, relationship_type=rel_type,
+                action=model.Action.READ)
+
+            if subregister != 'relationships' and not relationships:
+                return self._finish_not_found('Relationship "%s %s %s" not found.'\
+                                         % (id, subregister, id2))
+            
             response_data = [rel.as_dict(pkg1, ref_package_by=self.ref_package_by) for rel in relationships]
             return self._finish_ok(response_data)
         elif register == u'group':
-            group = model.Group.by_name(id)
+            group = self._get_group(id)
             if group is None:
                 response.status_int = 404
                 return ''
 
             if not self._check_access(group, model.Action.READ):
-                return ''
+                return self._finish_not_authz()
             for item in PluginImplementations(IGroupController):
                 item.read(group)
             _dict = group.as_dict(ref_package_by=self.ref_package_by)
             #TODO check it's not none
             return self._finish_ok(_dict)
         elif register == u'tag':
-            obj = model.Tag.by_name(id) #TODO tags
+            obj = self._get_tag(id) #TODO tags
             if obj is None:
                 response.status_int = 404
                 return ''            
             response_data = [pkgtag.package.name for pkgtag in obj.package_tags]
             return self._finish_ok(response_data)
-        elif register == u'harvestsource':
-            obj = model.HarvestSource.get(id, default=None)
-            if obj is None:
-                response.status_int = 404
-                return ''            
-            response_data = obj.as_dict()
-            return self._finish_ok(response_data)
-        elif register == u'harvesteddocument':
-            objs = model.Session.query(model.HarvestedDocument).filter(model.HarvestedDocument.guid==id)
-            if not objs:
-                response.status_int = 404
-                return ''            
-            obj = objs[0]
-            if not subregister:
-                response_data = obj.as_dict()
-                return self._finish_ok(response_data)
-            elif subregister == 'xml':
-                response_data = obj.as_dict()['content']
-                response.headers['Content-Type'] = 'text/xml'
-                response.status_int = 200
-                response.charset = 'utf-8'
-                return response_data
-            elif subregister == 'html':
-                d = obj.as_dict()
-                content = {
-                    'html': d['content'].replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;\n'),
-                    'title': d['id'],
-                }
-                response_data = '''
-                <html>
-                <head>
-                <title>Harvested Document %(title)s</title>
-                </head>
-                <body>
-                <h1>Harvested Document %(title)s</h1>
-                <pre style="background: #ffc">%(html)s</pre>
-                </body>
-                </html>
-                ''' % content
-                response.headers['Content-Type'] = 'text/html'
-                response.status_int = 200
-                response.charset = 'utf-8'
-                return response_data
-            else:
-                response.status_int = 404
-                return ''            
-        elif register == u'harvestingjob':
-            obj = model.HarvestingJob.get(id, default=None)
-            if obj is None:
-                response.status_int = 404
-                return ''            
-            response_data = obj.as_dict()
-            return self._finish_ok(response_data)
         else:
             response.status_int = 400
             return gettext('Cannot read entity of this type: %s') % register
@@ -329,7 +285,7 @@ class BaseRestController(BaseApiController):
         log.debug('create %s/%s/%s/%s params: %r' % (register, id, subregister, id2, request.params))
         # Check an API key given, otherwise deny access.
         if not self._check_access(None, None):
-            return json.dumps(_('Access denied'))
+            return self._finish_not_authz()
         # Read the request data.
         try:
             request_data = self._get_request_data()
@@ -338,16 +294,8 @@ class BaseRestController(BaseApiController):
             return gettext('JSON Error: %s') % str(inst)
         try:
             if register == 'package' and not subregister:
-                # Create a Package.
-                fs = self._get_standard_package_fieldset()
-                try:
-                    request_fa_dict = ckan.forms.edit_package_dict(ckan.forms.get_package_dict(fs=fs), request_data)
-                except ckan.forms.PackageDictFormatError, inst:
-                    log.error('Package format incorrect: %s' % str(inst))
-                    response.status_int = 400
-                    return gettext('Package format incorrect: %s') % str(inst)
-                fs = fs.bind(model.Package, data=request_fa_dict, session=model.Session)
-                # ...continues below.
+                # see /apiv1/PackageController
+                raise NotImplementedError
             elif register == 'package' and subregister in model.PackageRelationship.get_all_types():
                 # Create a Package Relationship.
                 pkg1 = self._get_pkg(id)
@@ -358,6 +306,11 @@ class BaseRestController(BaseApiController):
                 if not pkg2:
                     response.status_int = 404
                     return 'Second package named in address was not found.'
+                am_authorized = ckan.authz.Authorizer().\
+                                authorized_package_relationship(\
+                    c.user, pkg1, pkg2, action=model.Action.EDIT)
+                if not am_authorized:
+                    return self._finish_not_authz()
                 comment = request_data.get('comment', u'')
                 existing_rels = pkg1.get_relationships_with(pkg2, subregister)
                 if existing_rels:
@@ -372,6 +325,8 @@ class BaseRestController(BaseApiController):
                 return self._finish_ok(response_data)
             elif register == 'group' and not subregister:
                 # Create a Group.
+                if not self._check_access(model.System(), model.Action.GROUP_CREATE):
+                    return self._finish_not_authz()
                 is_admin = ckan.authz.Authorizer().is_sysadmin(c.user)
                 request_fa_dict = ckan.forms.edit_group_dict(ckan.forms.get_group_dict(), request_data)
                 fs = ckan.forms.get_group_fieldset(combined=True, is_admin=is_admin)
@@ -380,9 +335,6 @@ class BaseRestController(BaseApiController):
             elif register == 'rating' and not subregister:
                 # Create a Rating.
                 return self._create_rating(request_data)
-            elif register == 'harvestingjob' and not subregister:
-                # Create a HarvestingJob.
-                return self._create_harvesting_job(request_data)
             else:
                 # Complain about unsupported entity type.
                 log.error('Cannot create new entity of this type: %s %s' % (register, subregister))
@@ -409,8 +361,8 @@ class BaseRestController(BaseApiController):
             model.setup_default_user_roles(fs.model, admins)
             # Commit
             if register == 'package' and not subregister:
-                for item in PluginImplementations(IPackageController):
-                    item.create(fs.model)
+                # see /apiv1/PackageController
+                raise NotImplementedError
             elif register == 'group' and not subregister:
                 for item in PluginImplementations(IGroupController):
                     item.create(fs.model)
@@ -436,13 +388,11 @@ class BaseRestController(BaseApiController):
         log.debug('update %s/%s/%s/%s' % (register, id, subregister, id2))
         # must be logged in to start with
         if not self._check_access(None, None):
-            return json.dumps(_('Access denied'))
+            return self._finish_not_authz()
 
         if register == 'package' and not subregister:
-            entity = self._get_pkg(id)
-            if entity == None:
-                response.status_int = 404
-                return 'Package was not found.'
+            # see /apiv1/PackageController
+            raise NotImplementedError
         elif register == 'package' and subregister in model.PackageRelationship.get_all_types():
             pkg1 = self._get_pkg(id)
             pkg2 = self._get_pkg(id2)
@@ -452,16 +402,23 @@ class BaseRestController(BaseApiController):
             if not pkg2:
                 response.status_int = 404
                 return 'Second package named in address was not found.'
+            am_authorized = ckan.authz.Authorizer().\
+                            authorized_package_relationship(\
+                c.user, pkg1, pkg2, action=model.Action.EDIT)
+            if not am_authorized:
+                return self._finish_not_authz()
             existing_rels = pkg1.get_relationships_with(pkg2, subregister)
             if not existing_rels:
                 response.status_int = 404
                 return 'This relationship between the packages was not found.'
             entity = existing_rels[0]
         elif register == 'group' and not subregister:
-            entity = model.Group.by_name(id)
+            entity = self._get_group(id)
             if entity == None:
                 response.status_int = 404
                 return 'Group was not found.'
+            if not self._check_access(entity, model.Action.EDIT):
+                return self._finish_not_authz()
         else:
             response.status_int = 400
             return gettext('Cannot update entity of this type: %s') % register
@@ -472,7 +429,7 @@ class BaseRestController(BaseApiController):
         if (not subregister and \
             not self._check_access(entity, model.Action.EDIT)) \
             or not self._check_access(None, None):
-            return json.dumps(_('Access denied'))
+            return self._finish_not_authz()
 
         try:
             request_data = self._get_request_data()
@@ -482,13 +439,8 @@ class BaseRestController(BaseApiController):
 
         if not subregister:
             if register == 'package':
-                fs = self._get_standard_package_fieldset()
-                try:
-                    request_fa_dict = ckan.forms.GetEditFieldsetPackageData(
-                        fieldset=fs, package=entity, data=request_data).data
-                except ckan.forms.PackageDictFormatError, inst:
-                    response.status_int = 400
-                    return gettext('Package format incorrect: %s') % str(inst)
+                # see /apiv1/PackageController
+                raise NotImplementedError
             elif register == 'group':
                 auth_for_change_state = ckan.authz.Authorizer().am_authorized(c, 
                     model.Action.CHANGE_STATE, entity)
@@ -534,22 +486,10 @@ class BaseRestController(BaseApiController):
         log.debug('delete %s/%s/%s/%s' % (register, id, subregister, id2))
         # must be logged in to start with
         if not self._check_access(None, None):
-            return json.dumps(_('Access denied'))
-        # XXX Currently no security on this call
-        if register == 'harvestsource' and not subregister:
-            source = model.HarvestSource.get(id, default=None)
-            jobs = model.HarvestingJob.filter(source=source)
-            for job in jobs:
-                job.delete()
-            source.delete()
-            model.repo.commit()        
-            return self._finish_ok()
+            return self._finish_not_authz()
         if register == 'package' and not subregister:
-            entity = self._get_pkg(id)
-            if not entity:
-                response.status_int = 404
-                return 'Package was not found.'
-            revisioned_details = 'Package: %s' % entity.name
+            # see /apiv1/PackageController
+            raise NotImplementedError
         elif register == 'package' and subregister in model.PackageRelationship.get_all_types():
             pkg1 = self._get_pkg(id)
             pkg2 = self._get_pkg(id2)
@@ -559,6 +499,11 @@ class BaseRestController(BaseApiController):
             if not pkg2:
                 response.status_int = 404
                 return 'Second package named in address was not found.'
+            am_authorized = ckan.authz.Authorizer().\
+                            authorized_package_relationship(\
+                c.user, pkg1, pkg2, action=model.Action.EDIT)
+            if not am_authorized:
+                return self._finish_not_authz()
             existing_rels = pkg1.get_relationships_with(pkg2, subregister)
             if not existing_rels:
                 response.status_int = 404
@@ -566,14 +511,11 @@ class BaseRestController(BaseApiController):
             entity = existing_rels[0]
             revisioned_details = 'Package Relationship: %s %s %s' % (id, subregister, id2)
         elif register == 'group' and not subregister:
-            entity = model.Group.by_name(id)
+            entity = self._get_group(id)
             if not entity:
                 response.status_int = 404
                 return 'Group was not found.'
             revisioned_details = 'Group: %s' % entity.name
-        elif register == 'harvestingjob' and not subregister:
-            entity = model.HarvestingJob.get(id, default=None)
-            revisioned_details = None
         else:
             response.status_int = 400
             return gettext('Cannot delete entity of this type: %s %s') % (register, subregister or '')
@@ -581,15 +523,15 @@ class BaseRestController(BaseApiController):
             response.status_int = 404
             return ''
         if not self._check_access(entity, model.Action.PURGE):
-            return json.dumps(_('Access denied'))
+            return self._finish_not_authz()
         if revisioned_details:
             rev = model.repo.new_revision()
             rev.author = self.rest_api_user
             rev.message = _(u'REST API: Delete %s') % revisioned_details
         try:
             if register == 'package' and not subregister:
-                for item in PluginImplementations(IPackageController):
-                    item.delete(entity)
+                # see /apiv1/PackageController
+                raise NotImplementedError
             elif register == 'group' and not subregister:
                 for item in PluginImplementations(IGroupController):
                     item.delete(entity)
@@ -643,7 +585,7 @@ class BaseRestController(BaseApiController):
                 if (k in DEFAULT_OPTIONS.keys()):
                     options[k] = v
             options.update(params)
-            options.username = self._get_username()
+            options.username = c.user
             options.search_tags = False
             options.return_objects = False
             
@@ -746,37 +688,6 @@ class BaseRestController(BaseApiController):
                     'rating count': len(package.ratings)}
         return self._finish_ok(ret_dict)
 
-    def _create_harvesting_job(self, params):
-        """ Example data: {'user_ref':u'0005', 'source_id':5}
-        """
-        # Pick out attribute values from request.
-        user_ref = params.get('user_ref')
-        source_id = params.get('source_id')
-        # Validate values.
-        opts_err = ''
-        if not user_ref:
-            opts_err = gettext('You must supply a user_ref.')
-        elif not source_id:
-            opts_err = gettext('You must supply a source_id.')
-        else:
-            source = model.HarvestSource.get(source_id, default=None)
-            if not source:
-                opts_err = gettext('Harvest source %s does not exist.') % source_id
-        if opts_err:
-            self.log.debug(opts_err)
-            response.status_int = 400
-            response.headers['Content-Type'] = self.content_type_json
-            return json.dumps(opts_err)
-        # Create job.
-        job = model.HarvestingJob(source_id=source_id, user_ref=user_ref)
-        model.Session.add(job)
-        model.Session.commit()
-        ret_dict = job.as_dict()
-        return self._finish_ok(ret_dict)
-
-    def _get_username(self):
-        user = self._get_user_for_apikey()
-        return user and user.name or u''
 
     def _check_access(self, entity, action):
         # Checks apikey is okay and user is authorized to do the specified
@@ -787,11 +698,10 @@ class BaseRestController(BaseApiController):
         # Todo: Remove unused 'isOk' variable.
         isOk = False
 
-        self.rest_api_user = self._get_username()
+        self.rest_api_user = c.user
         log.debug('check access - user %r' % self.rest_api_user)
         
-        if action and entity and not isinstance(entity, model.PackageRelationship) \
-                and not isinstance(entity, model.HarvestingJob):
+        if action and entity and not isinstance(entity, model.PackageRelationship):
             if action != model.Action.READ and self.rest_api_user in (model.PSEUDO_USER__VISITOR, ''):
                 self.log.debug("Valid API key needed to make changes")
                 response.status_int = 403
@@ -812,7 +722,7 @@ class BaseRestController(BaseApiController):
         self.log.debug("Access OK.")
         response.status_int = 200
         return True                
-
+    
     def _update_package_relationship(self, relationship, comment):
         is_changed = relationship.comment != comment
         if is_changed:
@@ -821,7 +731,9 @@ class BaseRestController(BaseApiController):
             rev.message = _(u'REST API: Update package relationship: %s %s %s') % (relationship.subject, relationship.type, relationship.object)
             relationship.comment = comment
             model.repo.commit_and_remove()
-        return self._finish_ok(relationship.as_dict())
+        rel_dict = relationship.as_dict(package=relationship.subject,
+                                        ref_package_by=self.ref_package_by)
+        return self._finish_ok(rel_dict)
 
 
 class RestController(ApiVersion1, BaseRestController):
