@@ -44,12 +44,12 @@ class ManageDb(CkanCommand):
     db init # create and put in default data
     db clean
     db upgrade [{version no.}] # Data migrate
-    db dump {file-path} # dump to a file (json)
+    db dump {file-path} # dump to a pg_dump file
     db dump-rdf {package-name} {file-path}
     db simple-dump-csv {file-path}
     db simple-dump-json {file-path}
     db send-rdf {talis-store} {username} {password}
-    db load {file-path} # load a dump from a file
+    db load {file-path} # load a pg_dump from a file
     db create-from-model # create database from the model (indexes not made)
     db migrate06
     db migrate09a
@@ -120,22 +120,65 @@ class ManageDb(CkanCommand):
             print 'Command %s not recognized' % cmd
             sys.exit(1)
 
+    def _get_db_config(self):
+        from pylons import config
+        url = config['sqlalchemy.url']
+        # e.g. 'postgres://tester:pass@localhost/ckantest3'
+        db_details_match = re.match('^\s*(?P<db_type>\w*)://(?P<db_user>\w*):?(?P<db_pass>[^@]*)@(?P<db_host>[^/:]*):?(?P<db_port>[^/]*)/(?P<db_name>[\w.-]*)', url)
+        if not db_details_match:
+            raise Exception('Could not extract db details from url: %r' % url)
+        db_details = db_details_match.groupdict()
+        return db_details
+
+    def _get_postgres_cmd(self, command):
+        self.db_details = self._get_db_config()
+        if self.db_details.get('db_type') not in ('postgres', 'postgresql'):
+            raise AssertionError('Expected postgres database - not %r' % self.db_details.get('db_type'))
+        pg_cmd = command
+        pg_cmd += ' -U %(db_user)s' % self.db_details
+        if self.db_details.get('db_pass') not in (None, ''):
+            pg_cmd = 'export PGPASSWORD=%(db_pass)s && ' % self.db_details + pg_cmd
+        if self.db_details.get('db_host') not in (None, '', 'localhost'):
+            pg_cmd += ' -h %(db_host)s' % self.db_details
+        if self.db_details.get('db_port') not in (None, ''):
+            pg_cmd += ' -p %(db_port)s' % self.db_details
+        return pg_cmd
+
+    def _get_psql_cmd(self):
+        psql_cmd = self._get_postgres_cmd('psql')
+        psql_cmd += ' -d %(db_name)s' % self.db_details
+        return psql_cmd
+
+    def _postgres_dump(self, filepath):
+        pg_dump_cmd = self._get_postgres_cmd('pg_dump')
+        pg_dump_cmd += ' %(db_name)s' % self.db_details
+        pg_dump_cmd += ' > %s' % filepath
+        self._run_cmd(pg_dump_cmd)
+
+    def _postgres_load(self, filepath):
+        pg_cmd = self._get_psql_cmd() + ' -f %s' % filepath
+        self._run_cmd(pg_cmd)
+
+    def _run_cmd(self, command_line):
+        import subprocess
+        print command_line
+        subprocess.call(command_line, shell=True)
+
     def dump_or_load(self, cmd):
-        print 'This functionality is mothballed for now.'
-        return
         if len(self.args) < 2:
-            print 'Need dump path'
+            print 'Need pg_dump filepath'
             return
         dump_path = self.args[1]
-        import ckan.lib.dumper
-        dumper = ckan.lib.dumper.Dumper()
-        verbose = (self.verbose >= 2)
+
+        psql_cmd = self._get_psql_cmd() + ' -f %s'
         if cmd == 'load':
-            dumper.load_json(dump_path, verbose=verbose)
+            pg_cmd = self._postgres_load(dump_path)
         elif cmd == 'dump':
-            dumper.dump_json(dump_path, verbose=verbose)
+            pg_cmd = self._postgres_dump(dump_path)
         else:
             print 'Unknown command', cmd
+        if cmd == 'load':
+            print 'Now remember you have to call \'db upgrade\'.'
 
     def simple_dump_csv(self, cmd):
         from ckan import model
@@ -215,6 +258,37 @@ class SearchIndexCommand(CkanCommand):
         else:
             print 'Command %s not recognized' % cmd
 
+class Notification(CkanCommand):
+    '''Send out modification notifications.
+    
+    In "replay" mode, an update signal is sent for each package in the database.
+
+    Usage:
+      notify replay                        - send out modification signals
+    '''
+
+    summary = __doc__.split('\n')[0]
+    usage = __doc__
+    max_args = 1
+    min_args = 0
+
+    def command(self):
+        self._load_config()
+        from ckan.model import Session, Package, DomainObjectOperation
+        from ckan.model.modification import DomainObjectModificationExtension
+
+        if not self.args:
+            # default to run
+            cmd = 'replay'
+        else:
+            cmd = self.args[0]
+        
+        if cmd == 'replay':
+            dome = DomainObjectModificationExtension()
+            for package in Session.query(Package):
+                dome.notify(package, DomainObjectOperation.changed)
+        else:
+            print 'Command %s not recognized' % cmd
 
 class Sysadmin(CkanCommand):
     '''Gives sysadmin rights to a named user
@@ -894,309 +968,3 @@ class Changes(CkanCommand):
                     msg = "A @%s:  %s" % (attr_name, repr(new_value))
                     print msg.encode('utf8')
 
- 
-class Notifications(CkanCommand):
-    '''Manage notifications
-
-    Usage:
-      notifications monitor                 - runs monitor, printing all notifications
-    '''
-
-    summary = __doc__.split('\n')[0]
-    usage = __doc__
-    max_args = 1
-    min_args = 1
-
-    def command(self):
-        self._load_config()
-        from ckan import model
-        
-        from pylons import config
-        if config.get('carrot_messaging_library') != 'pyamqplib':
-            print 'Carrot messaging library not configured to AMQP. Currently set to:', config.get('carrot_messaging_library')
-            sys.exit(1)
-        
-        cmd = self.args[0]
-        if cmd == 'monitor':
-            self.monitor()
-        else:
-            print 'Command %s not recognized' % cmd
-
-    def monitor(self):
-        from ckan.lib import monitor
-        monitor = monitor.Monitor()
-
-
-class Harvester(CkanCommand):
-    '''Harvests remotely mastered metadata
-
-    Usage:
-      harvester source {url} [{user-ref} [{publisher-ref}]]     
-        - create new harvest source
-
-      harvester rmsource {url}
-        - remove a harvester source (and associated jobs)
-
-      harvester sources                                 
-        - lists harvest sources
-
-      harvester job {source-id} [{user-ref}]
-        - create new harvesting job
-
-      harvester rmjob {job-id}
-        - remove a harvesting job
-  
-      harvester jobs
-        - lists harvesting jobs
-
-      harvester run
-        - runs harvesting jobs
-    '''
-
-    summary = __doc__.split('\n')[0]
-    usage = __doc__
-    max_args = 4
-    min_args = 0
-
-    def command(self):
-        self._load_config()
-        # Clear the 'No handlers could be found for logger "vdm"' warning message.
-        print ""
-        if len(self.args) == 0:
-            self.parser.print_usage()
-            sys.exit(1)
-        cmd = self.args[0]
-        if cmd == 'source':
-            if len(self.args) >= 2:
-                url = unicode(self.args[1])
-            else:
-                print self.usage
-                print 'Error, source url is not given.'
-                sys.exit(1)
-            if len(self.args) >= 3:
-                user_ref = unicode(self.args[2])
-            else:
-                user_ref = u''
-            if len(self.args) >= 4:
-                publisher_ref = unicode(self.args[3])
-            else:
-                publisher_ref = u''
-            self.register_harvest_source(url, user_ref, publisher_ref)
-        elif cmd == "rmsource":
-            url = unicode(self.args[1])
-            self.remove_harvest_source(url)
-        elif cmd == 'sources':
-            self.list_harvest_sources()
-        elif cmd == 'job':
-            if len(self.args) >= 2:
-                source_id = unicode(self.args[1])
-            else:
-                print self.usage
-                print 'Error, job source is not given.'
-                sys.exit(1)
-            if len(self.args) >= 3:
-                user_ref = unicode(self.args[2])
-            else:
-                user_ref = u''
-            self.register_harvesting_job(source_id, user_ref)
-        elif cmd == "rmjob":
-            job_id = unicode(self.args[1])
-            self.remove_harvesting_job(job_id)
-        elif cmd == 'jobs':
-            self.list_harvesting_jobs()
-        elif cmd == 'run':
-            self.run_harvester()
-        else:
-            print 'Command %s not recognized' % cmd
-
-    def _load_config(self):
-        super(Harvester, self)._load_config()
-        import logging
-        logging.basicConfig()
-        logger_vdm = logging.getLogger('vdm')
-        logger_vdm.setLevel(logging.ERROR)
-
-    def run_harvester(self, *args, **kwds):
-        from pylons.i18n.translation import _get_translator
-        import pylons
-        pylons.translator._push_object(_get_translator(pylons.config.get('lang')))
-
-        from ckan.model import HarvestingJob
-        from ckan.controllers.harvesting import HarvestingJobController
-        from ckanext.csw.validation import Validator
-
-        jobs = HarvestingJob.filter(status=u"New").all()
-        jobs_len = len(jobs)
-        jobs_count = 0
-        if jobs_len:
-            print "Running %s harvesting jobs..." % jobs_len
-            profiles = [
-                x.strip() for x in
-                pylons.config.get(
-                    "ckan.harvestor.validator.profiles", 
-                    "iso19139,gemini2",
-                ).split(",")
-            ]
-            validator = Validator(profiles=profiles)
-            print ""
-            for job in jobs:
-                jobs_count += 1
-                if job.source is None:
-                    print 'ERRROR: no source associated with this job'
-                else:
-                    print "Running job %s/%s: %s" % (jobs_count, jobs_len, job.id)
-                    self.print_harvesting_job(job)
-                    job_controller = HarvestingJobController(job, validator)
-                    job_controller.harvest_documents()
-                    pprint (job.report)
-        else:
-            print "There are no new harvesting jobs."
-
-    def remove_harvesting_job(self, job_id):
-        from ckan import model
-        try:
-            job = model.HarvestingJob.get(job_id)
-            job.delete()
-            model.repo.commit_and_remove()
-            print "Removed job: %s" % job_id
-        except model.HarvestingObjectNotFound:
-            print "No such job"
-
-    def register_harvesting_job(self, source_id, user_ref):
-        from ckan.model import HarvestSource
-        from ckan.model import HarvestingJob
-        if re.match('(http|file)://', source_id):
-            source_url = unicode(source_id)
-            source_id = None
-            sources = HarvestSource.filter(url=source_url).all()
-            if sources:
-                source = sources[0]
-            else:
-                source = self.create_harvest_source(url=source_url, user_ref=user_ref, publisher_ref=u'')
-        else:
-            source = HarvestSource.get(source_id)
-        objects = HarvestingJob.filter(status='New', source=source)
-        if objects.count():
-            raise Exception('There is already an unrun job for the harvest source %r'%source.id)
-        job = HarvestingJob(
-            source=source,
-            user_ref=user_ref,
-            status=u"New",
-        )
-        job.save()
-        print "Created new harvesting job:"
-        self.print_harvesting_job(job)
-        status = u"New"
-        jobs = HarvestingJob.filter(status=status).all()
-        self.print_there_are("harvesting job", jobs, condition=status)
-
-    def register_harvest_source(self, url, user_ref, publisher_ref):
-        from ckan.model import HarvestSource
-        existing = self.get_harvest_sources(url=url)
-        if existing:
-            print "Error, there is already a harvesting source for that URL"
-            self.print_harvest_sources(existing)
-            sys.exit(1)
-        else:
-            source = self.create_harvest_source(url=url, user_ref=user_ref, publisher_ref=publisher_ref)
-            self.register_harvesting_job(source.id, user_ref)
-            print "Created new harvest source:"
-            self.print_harvest_source(source)
-            sources = self.get_harvest_sources()
-            self.print_there_are("harvest source", sources)
-
-    def remove_harvest_source(self, url):
-        from ckan import model
-        model.repo.new_revision()
-        sources = model.HarvestSource.filter(url=url)
-        if sources.count() == 0:
-            print "No such source"
-        else:
-            source = sources[0]
-            jobs = model.HarvestingJob.filter(source=source)
-            print "Removing %d jobs" % jobs.count()
-            for job in jobs:
-                job.delete()
-            source.delete()
-            model.repo.commit_and_remove()
-            print "Removed harvest source: %s" % url
-
-    def list_harvest_sources(self):
-        sources = self.get_harvest_sources()
-        self.print_harvest_sources(sources)
-        self.print_there_are(what="harvest source", sequence=sources)
-       
-    def list_harvesting_jobs(self):
-        jobs = self.get_harvesting_jobs()
-        self.print_harvesting_jobs(jobs)
-        self.print_there_are(what="harvesting job", sequence=jobs)
-
-    def get_harvest_sources(self, **kwds):
-        from ckan.model import HarvestSource
-        return HarvestSource.filter(**kwds).all()
-
-    def get_harvesting_jobs(self, **kwds):
-        from ckan.model import HarvestingJob
-        return HarvestingJob.filter(**kwds).all()
-
-    def create_harvest_source(self, **kwds):
-        from ckan.model import HarvestSource
-        source = HarvestSource(**kwds)
-        source.save()
-        return source
-
-    def create_harvesting_job(self, **kwds):
-        from ckan.model import HarvestingJob
-        job = HarvestingJob(**kwds)
-        job.save()
-        return job
-
-    def print_harvest_sources(self, sources):
-        if sources:
-            print ""
-        for source in sources:
-            self.print_harvest_source(source)
-
-    def print_harvest_source(self, source):
-        print "Source id: %s" % source.id
-        print "      url: %s" % source.url
-        print "     user: %s" % source.user_ref
-        print "publisher: %s" % source.publisher_ref
-        print "     docs: %s" % len(source.documents)
-        print ""
-
-    def print_harvesting_jobs(self, jobs):
-        if jobs:
-            print ""
-        for job in jobs:
-            self.print_harvesting_job(job)
-
-    def print_harvesting_job(self, job):
-        print "Job id: %s" % job.id
-        if job.user_ref:
-            print "  user: %s" % job.user_ref
-        print "status: %s" % job.status
-        print "source: %s" % job.source.id
-        print "   url: %s" % job.source.url
-        #print "report: %s" % job.report
-        if job.report and job.report['added']:
-            for package_id in job.report['added']:
-                print "   doc: %s" % package_id
-        if job.report and job.report['errors']:
-            for msg in job.report['errors']:
-                print " error: %s" % msg
-        print ""
-
-    def print_there_are(self, what, sequence, condition=""):
-        is_singular = self.is_singular(sequence)
-        print "There %s %s %s%s%s" % (
-            is_singular and "is" or "are",
-            len(sequence),
-            condition and ("%s " % condition.lower()) or "",
-            what,
-            not is_singular and "s" or "",
-        )
-
-    def is_singular(self, sequence):
-        return len(sequence) == 1
-    
