@@ -21,14 +21,15 @@ from resource import *
 from rating import *
 from package_relationship import *
 from changeset import Changeset, Change, Changemask
-from harvesting import HarvestSource, HarvestingJob, HarvestedDocument
 import ckan.migration
+from ckan.lib.helpers import OrderedDict
 
 # set up in init_model after metadata is bound
 version_table = None
 
 def init_model(engine):
     '''Call me before using any of the tables or classes in the model'''
+    meta.Session.remove()
     meta.Session.configure(bind=engine)
     meta.engine = engine
     meta.metadata.bind = engine
@@ -177,12 +178,12 @@ repo = Repository(metadata, Session,
 def _get_packages(self):
     changes = repo.list_changes(self)
     pkgs = set()
-    for pkg_rev in changes.pop(Package):
-        pkgs.add(pkg_rev.continuity)
-    for non_pkg_rev_list in changes.values():
-        for non_pkg_rev in non_pkg_rev_list:
-            if hasattr(non_pkg_rev.continuity, 'package'):
-                pkgs.add(non_pkg_rev.continuity.package)
+    for revision_list in changes.values():
+        for revision in revision_list:
+            obj = revision.continuity
+            if hasattr(obj, 'related_packages'):
+                pkgs.update(obj.related_packages())
+
     return list(pkgs)
 
 def _get_groups(self):
@@ -213,3 +214,14 @@ def strptimestamp(s):
 def strftimestamp(t):
     return t.isoformat()
 
+def revision_as_dict(revision, include_packages=True, ref_package_by='name'):
+    revision_dict = OrderedDict((
+        ('id', revision.id),
+        ('timestamp', strftimestamp(revision.timestamp)),
+        ('message', revision.message),
+        ('author', revision.author),
+        ))
+    if include_packages:
+        revision_dict['packages'] = [getattr(pkg, ref_package_by) \
+                                     for pkg in revision.packages]
+    return revision_dict
