@@ -1,10 +1,12 @@
 from webob.multidict import UnicodeMultiDict, MultiDict
+from nose.tools import assert_raises, assert_equal
 
 from ckan.tests import *
 from ckan.tests import is_search_supported
 from ckan.lib.search import get_backend, QueryOptions
 from ckan import model
 from ckan.lib.create_test_data import CreateTestData
+from ckan.lib.search.common import SearchError
 
 class TestSearch(object):
     @classmethod
@@ -21,16 +23,23 @@ class TestSearch(object):
                  {'url':self.ab,
                   'description':'This is site ab.',
                   'format':'Excel spreadsheet',
-                  'hash':'abc-123'},
+                  'hash':'abc-123',
+                  'alt_url': 'alt1',
+                  'extras':{'size': '100'},
+                  },
                  {'url':self.cd,
                   'description':'This is site cd.',
                   'format':'Office spreadsheet',
-                  'hash':'qwe-456'},
+                  'hash':'qwe-456',
+                  'alt_url':'alt2',
+                  'extras':{'size':'200'},
+                  },
                  ]             
              },
             {'name':'pkg2',
              'resources':[
                  {'url':self.cd,
+                  'alt_url': 'alt1',
                   'description':'This is site cd.'},
                  {'url':self.ef,
                   'description':'This is site ef.'},
@@ -49,16 +58,16 @@ class TestSearch(object):
         CreateTestData.delete()
 
     def res_search(self, query='', fields={}, terms=[], options=QueryOptions()):
-        result = self.backend.query_for(model.PackageResource).run(query=query, fields=fields, terms=terms, options=options)
-        resources = [model.Session.query(model.PackageResource).get(resource_id) for resource_id in result['results']]
+        result = self.backend.query_for(model.Resource).run(query=query, fields=fields, terms=terms, options=options)
+        resources = [model.Session.query(model.Resource).get(resource_id) for resource_id in result['results']]
         urls = set([resource.url for resource in resources])
         return urls
 
     def test_01_search_url(self):
         fields = {'url':'site.com'}
-        result = self.backend.query_for(model.PackageResource).run(fields=fields)
+        result = self.backend.query_for(model.Resource).run(fields=fields)
         assert result['count'] == 6, result
-        resources = [model.Session.query(model.PackageResource).get(resource_id) for resource_id in result['results']]
+        resources = [model.Session.query(model.Resource).get(resource_id) for resource_id in result['results']]
         urls = set([resource.url for resource in resources])
         assert set([self.ab, self.cd, self.ef]) == urls, urls
         
@@ -107,14 +116,14 @@ class TestSearch(object):
     def test_12_search_all_fields(self):
         fields = {'url':'a/b'}
         options = QueryOptions(all_fields=True)
-        result = self.backend.query_for(model.PackageResource).run(fields=fields, options=options)
+        result = self.backend.query_for(model.Resource).run(fields=fields, options=options)
         assert result['count'] == 1, result
         res_dict = result['results'][0]
         assert isinstance(res_dict, dict)
         res_keys = set(res_dict.keys())
-        expected_res_keys = model.PackageResource.get_columns() + \
-                            ['id', 'package_id', 'position']
-        assert res_keys == set(expected_res_keys), res_keys
+        expected_res_keys = set(model.Resource.get_columns())
+        expected_res_keys.update(['id', 'resource_group_id', 'package_id', 'position', 'size'])
+        assert_equal(res_keys, expected_res_keys)
         pkg1 = model.Package.by_name(u'pkg1')
         ab = pkg1.resources[0]
         assert res_dict['id'] == ab.id
@@ -129,7 +138,7 @@ class TestSearch(object):
         # large search
         options = QueryOptions(order_by='hash')
         fields = {'url':'site'}
-        all_results = self.backend.query_for(model.PackageResource).run(fields=fields, options=options)
+        all_results = self.backend.query_for(model.Resource).run(fields=fields, options=options)
         all_resources = all_results['results']
         all_resource_count = all_results['count']
         assert all_resource_count >= 6, all_results
@@ -137,7 +146,7 @@ class TestSearch(object):
         # limit
         options = QueryOptions(order_by='hash')
         options.limit = 2
-        result = self.backend.query_for(model.PackageResource).run(fields=fields, options=options)
+        result = self.backend.query_for(model.Resource).run(fields=fields, options=options)
         resources = result['results']
         count = result['count']
         assert len(resources) == 2, resources
@@ -148,7 +157,7 @@ class TestSearch(object):
         options = QueryOptions(order_by='hash')
         options.limit = 2
         options.offset = 2
-        result = self.backend.query_for(model.PackageResource).run(fields=fields, options=options)
+        result = self.backend.query_for(model.Resource).run(fields=fields, options=options)
         resources = result['results']
         assert len(resources) == 2, resources
         assert resources == all_resources[2:4]
@@ -157,7 +166,24 @@ class TestSearch(object):
         options = QueryOptions(order_by='hash')
         options.limit = 2
         options.offset = 4
-        result = self.backend.query_for(model.PackageResource).run(fields=fields, options=options)
+        result = self.backend.query_for(model.Resource).run(fields=fields, options=options)
         resources = result['results']
         assert len(resources) == 2, resources
         assert resources == all_resources[4:6]
+
+    def test_14_extra_info(self):
+
+        fields = {'alt_url':'alt1'}
+        result = self.backend.query_for(model.Resource).run(fields=fields)
+        assert result['count'] == 2, result
+
+        fields = {'alt_url':'alt2'}
+        result = self.backend.query_for(model.Resource).run(fields=fields)
+        assert result['count'] == 1, result
+
+        # Document that resource extras not in ckan.extra_resource_fields
+        # can't be searched
+        fields = {'size':'100'}
+        assert_raises(SearchError, self.backend.query_for(model.Resource).run, fields=fields)
+
+
